Mahalanobis squared distances of Brownlee's stack loss plant operation data based only on the explanatory variates (air flow, water temperature, and acid concentration).
Source:R/stacklossDistances.R
stacklossDistances.RdMahalanobis distances were calculated using the mahalanobis R function. Under standard normal theory, these are approximately Chi-squared on 3 degrees of freedom.
Format
A data frame with 21 rows and 2 variates:
- ordinary
Mahalanobis squared distances from the arithemetic mean using the eliptical contours of the sample covariance matrix.
- robust
As with distances, these are Mahalobis squared distances butnow based on robust measures of location and covariance matrix (as determined from the default covRob of the robust package).