hideLocation: 'hideLocation' Obfuscating the true location of the real...

View source: R/hideLocation.R

hideLocationR Documentation

hideLocation Obfuscating the true location of the real data.

Description

Hides the true location of the data as a non-obvious calculation string.

Usage

hideLocation(trueLoc, nSubjects)

Arguments

trueLoc

A vector of one or more numbers in the set {1, 2, ..., nSubjects} whose locations are to be hidden.

nSubjects

An integer larger than 1 used to define the set {1, 2, ..., nSubjects}.

Value

Returns a character vector, each element being a string containing an obscure calculation which, if parsed and evaluated, would return the value of the corresponding number in trueLoc.

See Also

revealLocation

Examples

trueLoc <- hideLocation(3,100)
trueLoc
revealLocation(trueLoc)

n <- 200
trueLoc <- sample(1:n, 3)
trueLoc
ans <- hideLocation(trueLoc , n)
ans
revealLocation(ans)


qqtest documentation built on July 22, 2026, 1:07 a.m.