| hideLocation | R Documentation |
hideLocation Obfuscating the true location of the real data.Hides the true location of the data as a non-obvious calculation string.
hideLocation(trueLoc, nSubjects)
trueLoc |
A vector of one or more numbers in the set
{1, 2, ..., |
nSubjects |
An integer larger than 1 used to define the set
{1, 2, ..., |
Returns a character vector, each element being a string
containing an obscure calculation which, if parsed and evaluated, would return
the value of the corresponding number in trueLoc.
revealLocation
trueLoc <- hideLocation(3,100)
trueLoc
revealLocation(trueLoc)
n <- 200
trueLoc <- sample(1:n, 3)
trueLoc
ans <- hideLocation(trueLoc , n)
ans
revealLocation(ans)
Add the following code to your website.
For more information on customizing the embed code, read Embedding Snippets.