hideLocation: hideLocation Obfuscating the true location of the real data.
Description
Hides the true location of the data as a non-obvious calculation string.
Usage
hideLocation(trueLoc, nSubjects)
Value
Returns a character vector, each element being a string
containing an obscure calculation which, if parsed and evaluated, would return
the value of the corresponding number in trueLoc.
Arguments
trueLoc
A vector of one or more numbers in the set
{1, 2, ..., nSubjects} whose locations are to be hidden.
nSubjects
An integer larger than 1 used to define the set
{1, 2, ..., nSubjects}.